from kooki.steps import Extension, Metadata, Document, Template

def recipe(document, config):
    extensions = Extension(config, filters=['.svg'])
    metadata = Metadata(config, metadata=document.metadata)
    document_content = Document(config, template=document.template, extensions=extensions, metadata={**extensions, **metadata})
    name = Template(document.name, metadata)

    Export(name, document_content, 'svg')



import subprocess, os, shutil

from kooki.tools import write_file
from kooki.tools.output import Output
from kooki import DocumentException


def Xelatex(config, name, document):

    Output.start_step('pdf')

    tex_file_name = '{0}.tex'.format(name)
    pdf_file_name = '{0}.pdf'.format(name)

    tex_file_path = os.path.join(config.temp_dir, tex_file_name)
    pdf_file_path = os.path.join(config.temp_dir, pdf_file_name)

    write_file(tex_file_path, document)

    command = 'xelatex -interaction=nonstopmode -halt-on-error -output-directory={1} {0}'.format(tex_file_name, config.temp_dir)
    log_file = os.path.join(config.temp_dir, 'xelatex.log')

    with open(log_file, "w") as f:
        subprocess.call(command, shell=True, stdout=f)
        subprocess.call(command, shell=True, stdout=f)

    if Output.debug:
        Output.info('XeLaTeX output: cat {0}'.format(log_file))

    if os.path.isfile(pdf_file_path):
        shutil.copy(pdf_file_path, os.getcwd())
        Output.info('PDF generated: {0}'.format(pdf_file_name))
    else:
        raise DocumentException('pdf is missing')



from kooki.tools import write_file
from kooki.tools.output import Output

def Export(name, content, extension):
    Output.start_step('export')
    file_name = '{0}.{1}'.format(name, extension)
    Output.info(file_name)
    write_file(file_name, content)



from kooki.kooki import ContentKooki
from kooki.tools.output import Output
from kooki import DocumentException

def Content(contents, utensils):
    Output.start_step('contents')
    result = []
    infos = []

    for content in contents:
        kooki = ContentKooki(content)
        for utensil in utensils:
            kooki.add_utensil(utensil)
        kooki_cooked = kooki.cook()
        result.append(kooki_cooked['content'])

    return result



from kooki.kooki import Kooki
from kooki.tools.output import Output
from kooki import DocumentException

from . import Step

def Content(config, files, utensils):
    step = _Content(config, utensils)
    return step(files)


class _Content(Step):

    def __init__(self, config, utensils):
        super().__init__(config)
        self._infos = []
        self.utensils = utensils

    def __call__(self, files):

        content = ''
        error = False

        Output.start_step('contents')

        for source in files:
            try:
                kooki = self._load(source)
                kooki_cooked = kooki.cook()
                content += kooki_cooked['content']
                self._infos.append({'name': source, 'status': '[found]', 'info': ''})

            except DocumentException as e:
                self._infos.append({'name': source, 'status': ('[error]', 'red'), 'info': (str(e), 'red')})
                error = True

            except NameError as e:
                self._infos.append({'name': source, 'status': ('[error]', 'red'), 'info': (str(e), 'red')})
                error = True

            except Exception as e:
                # self._infos.append({'name': source, 'status': ('[missing]', 'red')})
                raise Exception(type(e))

        if len(files) > 0:
            Output.infos(self._infos, [('name', 'cyan'), ('status', 'green'), ('info', 'cyan')])
        else:
            Output.info('no content')

        if error:
            raise DocumentException('Something went wrong... :(')

        return content

    def _load(self, content_source):

        kooki = Kooki(content_source)

        for utensil in self.utensils:
            kooki.add_utensil(utensil)

        return kooki



from kooki import DocumentException
from kooki.tools.output import Output

def Document(config, template, extensions, metadata):

    Output.start_step('document')
    Output.info(template)

    if template in extensions:
        kooki = extensions[template]
    else:
        raise DocumentException('bad template')

    document = kooki(**metadata)

    return document



from kooki.steps import Extension, Metadata, Document, Sass, Font, Js, Css, Template, Export, Split, Read
from kooki.steps.content2 import Content
from kooki.steps.merge import Merge
from kooki.steps.custom import Custom
from kooki.utensils import MarkdownToHTML, Empy

def custom(content, index):
    slide_number = index + 1
    classes = ''
    html_id = ''
    others = ''
    slide_content = content
    result = ''

    if content.startswith('<p>{:'):
        end = content.find('}</p>')
        attrs = content[5:end].split()
        slide_content = content[end + 5:]
        for attr in attrs:
            if attr.startswith('.'):
                classes += attr[1:]
            elif attr.startswith('#'):
                html_id = attr[1:]
            else:
                others += attr

    result = '''
<section id="{0}" class="slide slide-{3} {1}" {2}>
    <div class="slide-number">{3}</div>
    <div class="slide-content">
        <div class="slide-content-child">
        {4}
        </div>
    </div>
</section>
'''.format(html_id, classes, others, slide_number, slide_content)

    return result

def recipe(document, config):
    extensions = Extension(config, filters=['.html'])
    metadata = Metadata(config, document.metadata)
    fonts = Font(config)
    js = Js(config)
    css = Css(config)
    sass = Sass(config)

    text = Read(document.contents)
    parts = Split(text, '---\n')

    template = Empy({**extensions, **metadata})
    html = MarkdownToHTML()
    utensils = [template, html]

    result = Content(parts, utensils)
    result = Custom(result, custom)
    result = Merge(result)

    document_content = Document(config, document.template, extensions, metadata={
        **extensions,
        **metadata,
        'js': js,
        'css': css,
        'sass': sass,
        'fonts': fonts,
        'content': result})

    name = Template(document.name, metadata)

    Export(name, document_content, 'html')



from .step import Step
from .content import Content
from .extension import Extension
from .metadata import Metadata
from .document import Document
from .css import Css
from .font import Font
from .js import Js
from .sass import Sass
from .xelatex import Xelatex
from .template import Template
from .export import Export
from .split import Split
from .read import Read



import em
from kooki.tools import DictAsMember

def Template(text, metadata):
    interpreter = em.Interpreter()
    interpreter.setPrefix('@')
    metadata_member = DictAsMember.convert(metadata)
    result = interpreter.expand(text, metadata_member)
    return result



from kooki.tools.output import Output

def Split(text, separator):
    Output.start_step('split')
    parts = text.split('---\n')
    Output.info('text splitted {} times'.format(len(parts)))
    return parts



from kooki.kooki import ContentKooki
from kooki.tools.output import Output
from kooki import DocumentException

def Merge(parts):
    Output.start_step('merge')
    result = ''
    for part in parts:
        result += part
    return result
